#include "stm8s.h"
#include "stm8s.h"
#include "stm8s_gpio.h"
#include "stm8s_tim4.h"
#include "stm8s_flash.h"
#include "stm8s_exti.h"

#include "globals.h"
#include "ltkrn.h"

uint16_t *eeToken = (uint16_t*)FLASH_DATA_START_PHYSICAL_ADDRESS;
uint32_t *eeOdometer = (uint32_t *)(FLASH_DATA_START_PHYSICAL_ADDRESS + 2);
uint16_t *eeTripMeter = (uint16_t *)(FLASH_DATA_START_PHYSICAL_ADDRESS + 6);
uint16_t *eeSpeedCnt = (uint16_t *)(FLASH_DATA_START_PHYSICAL_ADDRESS + 8);

uint16_t voltages_adc[16];
uint8_t voltages_pos = 0;

void ee_Unlock()
{
  FLASH->DUKR = FLASH_RASS_KEY2; /* Warning: keys are reversed on data memory !!! */
  FLASH->DUKR = FLASH_RASS_KEY1;
}

void ee_Lock()
{
  /* Lock memory */
  FLASH->IAPSR &= (uint8_t)FLASH_MEMTYPE_DATA;
}

// ,         
void ee_CheckDefaultValuesAndRead()
{
  if (*eeToken != 0xAEEA)
  {
    ee_Unlock();
    *eeToken = 0xAEEA;
    *eeOdometer = 0;
    *eeTripMeter = 0;
    *eeSpeedCnt  = 0;
    ee_Lock();
  }
  else
  {
    speed_cnt = *eeSpeedCnt;
    odometer = *eeOdometer;
    trip_km = *eeTripMeter;   
    // ,       
    uint8_t i;
    uint16_t _trip_km = trip_km;
    uint16_t div = 10000;
    uint16_t res;
    for (i = 0; i < 5; i++)
    {
      res = _trip_km / div;
      trip_value[i] = (uint8_t) res;
      _trip_km = _trip_km - res * div;
      div = div / 10;
    }
    
    uint32_t _odometer = odometer;
    uint32_t div1 = 100000;
    uint32_t res1;
    for (i = 0; i < 6; i++)
    {
      res1 = _odometer / div1;
      odo_value[i] = (uint8_t) res1;
      _odometer = _odometer - res1 * div1;
      div1 = div1 / 10;      
    }
  }
}

void ee_Update_Values(uint32_t odo, uint16_t trip, uint16_t speed)
{
  ee_Unlock();
  if (odo != *eeOdometer)
    *eeOdometer = odo;
  if (trip != *eeTripMeter)
    *eeTripMeter = trip;
  if (speed != *eeSpeedCnt)
    *eeSpeedCnt  = speed;
  ee_Lock();
}


void init_hardware(void)
{
  // ---------------------------------------
  // clock system
  //CLK_DeInit();  
  CLK->ICKR = CLK_ICKR_RESET_VALUE;
  CLK->ECKR = CLK_ECKR_RESET_VALUE;
  CLK->SWR  = CLK_SWR_RESET_VALUE;
  CLK->SWCR = CLK_SWCR_RESET_VALUE;
  CLK->CKDIVR = CLK_CKDIVR_RESET_VALUE;
  CLK->PCKENR1 = CLK_PCKENR1_RESET_VALUE;
  CLK->PCKENR2 = CLK_PCKENR2_RESET_VALUE;
  CLK->CSSR = CLK_CSSR_RESET_VALUE;
  CLK->CCOR = CLK_CCOR_RESET_VALUE;
  while ((CLK->CCOR & CLK_CCOR_CCOEN)!= 0)
  {}
  CLK->CCOR = CLK_CCOR_RESET_VALUE;
  CLK->HSITRIMR = CLK_HSITRIMR_RESET_VALUE;
  CLK->SWIMCCR = CLK_SWIMCCR_RESET_VALUE;
  
  //CLK_SYSCLKConfig(CLK_PRESCALER_HSIDIV1);
  CLK->CKDIVR &= (uint8_t)(~CLK_CKDIVR_HSIDIV);
  CLK->CKDIVR |= (uint8_t)((uint8_t)CLK_PRESCALER_HSIDIV1 & (uint8_t)CLK_CKDIVR_HSIDIV);
  
  //CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSI, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE);
  CLK->SWCR |= CLK_SWCR_SWEN;
  CLK->SWCR &= (uint8_t)(~CLK_SWCR_SWIEN);
  CLK->SWR = (uint8_t)CLK_SOURCE_HSI;
  uint16_t DownCounter = CLK_TIMEOUT;
  while ((((CLK->SWCR & CLK_SWCR_SWBSY) != 0 )&& (DownCounter != 0)))
    DownCounter--;
  CLK->ICKR &= (uint8_t)(~CLK_ICKR_HSIEN);
  //----------------------------------------
  // 4   ,    1
  //CLK_PeripheralClockConfig(CLK_PERIPHERAL_TIMER4,ENABLE);
  CLK->PCKENR1 |= (uint8_t)((uint8_t)1 << ((uint8_t)CLK_PERIPHERAL_TIMER4 & (uint8_t)0x0F));  
  //TIM4_TimeBaseInit(TIM4_PRESCALER_128, 125 - 1);  
  /* Set the Prescaler value */
  TIM4->PSCR = (uint8_t)(TIM4_PRESCALER_128);
  /* Set the Autoreload value */
  TIM4->ARR = (uint8_t)(125 - 1);
  
  //TIM4_ClearFlag(TIM4_FLAG_UPDATE);
  TIM4->SR1 = (uint8_t)(~TIM4_FLAG_UPDATE);
  //TIM4_UpdateRequestConfig(TIM4_UPDATESOURCE_REGULAR);
  TIM4->CR1 |= TIM4_CR1_URS;
  //TIM4_ITConfig(TIM4_IT_UPDATE,ENABLE);
  TIM4->IER |= (uint8_t)TIM4_IT_UPDATE;
  //TIM4_Cmd(ENABLE);
  TIM4->CR1 |= TIM4_CR1_CEN;
  
 
  //----------------------------------------
  //  2   
  TIM2_TimeBaseInit(TIM2_PRESCALER_512, 65535); 
  TIM2_UpdateRequestConfig(TIM2_UPDATESOURCE_REGULAR);
  TIM2_ITConfig(TIM2_IT_UPDATE,ENABLE);  
  TIM2_Cmd(ENABLE);
  
  //----------------------------------------
  //   PD5/PD6  
  GPIO_Init(GPIOD,GPIO_PIN_5,GPIO_MODE_IN_PU_NO_IT); // BTN1
  GPIO_Init(GPIOD,GPIO_PIN_6,GPIO_MODE_IN_PU_NO_IT); // BTN2
  //----------------------------------------
  // PD3 AIN  
  // PD2 AIN  
  
  GPIO_Init(GPIOD,GPIO_PIN_3,GPIO_MODE_IN_FL_NO_IT); // AIN4
  GPIO_Init(GPIOD,GPIO_PIN_2,GPIO_MODE_IN_FL_NO_IT); // AIN3
  
  // ,    
  //ADC1_DeInit();
  //ADC1_ScanModeCmd(ENABLE);
  //ADC1_DataBufferCmd(ENABLE);
  //ADC1_Init(ADC1_CONVERSIONMODE_SINGLE, ADC1_CHANNEL_3 | ADC1_CHANNEL_4,
  //          ADC1_PRESSEL_FCPU_D8, ADC1_EXTTRIG_TIM, DISABLE, ADC1_ALIGN_RIGHT,
  //          ADC1_SCHMITTTRIG_CHANNEL3 | ADC1_SCHMITTTRIG_CHANNEL4, DISABLE);
  
  ADC1->CR1 = 0x41;
  ADC1->CR2 = 0x0A;
  ADC1->CR3 = 0x80;
  ADC1->CSR = 0x07;  
  ADC1->TDRH = 0x00;
  ADC1->TDRL = 0x80;
  ADC1->AWSRH = 0x00;
  ADC1->AWSRL = 0x00;
  ADC1->AWCRL = 0x00;
  
  // PA3   
  GPIO_Init(GPIOA,GPIO_PIN_3,GPIO_MODE_IN_PU_IT); //speed sensor in
  EXTI->CR1 &= (uint8_t)(~EXTI_CR1_PAIS);
  EXTI->CR1 |= (uint8_t)(EXTI_SENSITIVITY_FALL_ONLY);
  
  
  //----------------------------------------
  //GPIO_Init(GPIOC,GPIO_PIN_5,GPIO_MODE_OUT_PP_LOW_FAST); //SCK
  //GPIO_Init(GPIOC,GPIO_PIN_6,GPIO_MODE_OUT_PP_LOW_FAST); //MOSI -  
  //GPIO_Init(GPIOC,GPIO_PIN_3,GPIO_MODE_OUT_PP_HIGH_FAST); //CS
  
  GPIO_Init(GPIOD,GPIO_PIN_4,GPIO_MODE_OUT_PP_LOW_FAST); //LIGHT
  GPIO_Init(GPIOA,GPIO_PIN_1,GPIO_MODE_OUT_PP_HIGH_FAST); //RESET
  
  GPIOC->ODR = 0x10;
  GPIOC->IDR = 0x08;
  GPIOC->DDR = 0x78;
  GPIOC->CR1 = 0x78;
  GPIOC->CR2 = 0x78;
  
  ee_CheckDefaultValuesAndRead();
}

uint16_t ADC_GetBufferValue(uint8_t Buffer)
{
  
  uint16_t temph = 0;
  uint8_t templ = 0;
  /* Read LSB first */
  templ = *(uint8_t*)(uint16_t)((uint16_t)ADC1_BaseAddress + (uint8_t)(Buffer << 1) + 1);
  /* Then read MSB */
  temph = *(uint8_t*)(uint16_t)((uint16_t)ADC1_BaseAddress + (uint8_t)(Buffer << 1));
  
  temph = (uint16_t)(templ | (uint16_t)(temph << (uint8_t)8));
  
  return ((uint16_t)temph); 
}


INTERRUPT_HANDLER(TIM2_handler,ITC_IRQ_TIM2_OVF)
{
  TIM2->SR1 = (uint8_t)(~((uint8_t)(TIM2_IT_UPDATE)));
  last_speed_tmr = 0xFFFF;
}

INTERRUPT_HANDLER(TIM4_handler,ITC_IRQ_TIM4_OVF)
{
   
  krn_timer_warp --;
  if (krn_timer_warp == 0)
  {
    krn_timer_cnt++;
    krn_timer_current++;
    krn_timer_warp = 5;
    krn_dispatch();
    //GPIOA->ODR ^= GPIO_PIN_1;
  }
  
  TIM4->SR1 = (uint8_t)(~((uint8_t)(TIM4_IT_UPDATE)));
  
  sys_msec++;
  
  if((ADC1->CSR & 0x80)) //  ?
  {
    ADC1->CSR &= 0x7F; //      
    
    voltages_adc[voltages_pos++] = ADC_GetBufferValue(ADC1_CHANNEL_3);
    if (voltages_pos > 15)
      voltages_pos = 0;
    
    lpg_level = ADC_GetBufferValue(ADC1_CHANNEL_4);        
  }
  
  if (sys_msec == 1000)
  {
    //           
    ADC1->CR1 |= ADC1_CR1_ADON;
    sys_sec++;
    sys_msec = 0;
  }    
}

// ---------------------------------------
//      
INTERRUPT_HANDLER(PORTA_PA3_handler, ITC_IRQ_PORTA)
{
  last_speed_tmr = ((uint16_t)(TIM2->CNTRH)) << 8 | ((uint16_t) TIM2->CNTRL);
  TIM2->CNTRH = 0; TIM2->CNTRL = 0;
  speed_cnt++;
}

