#include "stm8s.h"
#include "stm8s_gpio.h"
#include "stm8s_tim4.h"

#include "ili9341.h"
#include "hardware.h"
#include "ltkrn.h"

#include "globals.h"

#define OS_THREAD_STACK 240
#define MAIN_THREAD_STACK 80

#define BUTTON_HOLD_TICK KRN_FREQ

#define BUTTON_1 0x01
#define BUTTON_1_MASK 0xfe
#define BUTTON_2 0x02
#define BUTTON_2_MASK 0xfd



#define button_thread_stack (void*)(KRN_STACKFRAME - 1 * MAIN_THREAD_STACK)
#define user_thread_stack (void*)(KRN_STACKFRAME - 2 * MAIN_THREAD_STACK)

static krn_thread thr_button, thr_user;


typedef struct
{
  uint8_t JustPressed; //    
  uint8_t JustReleased; //    
  uint8_t Pressed;  //     
  uint8_t Hold;  //     
  uint8_t count_btn1; //    1  2
  uint8_t count_btn2;
  uint8_t count_btn3;
  
} TBUTTONS;



volatile TBUTTONS buttons;

static NO_REG_SAVE void button_thread_func (void)
{
  static uint8_t prev_state = 0x00;
  uint8_t _temp;
  
  buttons.count_btn1 = 0;
  buttons.count_btn2 = 0;
  buttons.Pressed = 0;
  buttons.JustPressed = 0;
  buttons.JustReleased = 0;
  
  for (;;)
  {
    krn_sleep(KRN_FREQ/50); //   20
    
    _temp = 0x00; //  
    
    if (PIN_BUTTON_1)
      _temp |= 0x01;
    if (PIN_BUTTON_2)
      _temp |= 0x02;
    
    _temp ^= prev_state;	// xor   ,   
    prev_state ^= _temp; //  prev    
    
    if (_temp) //  
    {
      continue;
    }
    _temp = prev_state ^ buttons.Pressed; //    
    buttons.JustPressed |= (_temp & prev_state);
    if (_temp & buttons.Pressed) //      
    {
      buttons.JustReleased |= _temp & buttons.Pressed;
    }
    
    buttons.Pressed = prev_state; //   
    
    //   ,   1  2.
    if (buttons.Pressed & 0x01)
    {
      buttons.count_btn1++;
      if (buttons.count_btn1 >= BUTTON_HOLD_TICK)
      {
        buttons.Hold |= 0x01; //   
      }
    }
    else
    {
      buttons.count_btn1 = 0;
      buttons.Hold &= 0xfe; //   ,  
    }
    if (buttons.Pressed & 0x02)
    {
      buttons.count_btn2++;
      if (buttons.count_btn2 >= BUTTON_HOLD_TICK)
      {
        buttons.Hold |= 0x02; //   
      }
      
    }
    else
    {
      buttons.count_btn2 = 0;
      buttons.Hold &= 0xfd; //   ,  
    }
    if (buttons.Pressed & 0x04)
    {
      buttons.count_btn3++;
      if (buttons.count_btn3 >= BUTTON_HOLD_TICK)
      {
        buttons.Hold |= 0x04; //   
      }
      
    }
    else
    {
      buttons.count_btn3 = 0;
      buttons.Hold &= 0xfb; //   ,  
    }
    
  }
}


void convert(uint16_t value, uint8_t * data)
{
  uint16_t res, val;
  uint16_t divider = 1000;
  uint8_t i;
  val = value;
  for (i = 0; i < 3; i++)
  {
    res = val / divider;
    *data = (uint8_t) res;
    data++;
    val = val - res * divider;
    divider = divider / 10;
  }
  *data = val;
}

void increase_trip()
{
  trip_km++;
  uint8_t flag = 0;
  
  
  int8_t i;  
  trip_value[4]++; 
  if (trip_value[4] > 9)
  {
    odometer++;
    flag = 1;
  }
  
  for (i = 4; i>0; i--)
  {
    if (trip_value[i] > 9)
    {
      trip_value[i] = 0;
      trip_value[i-1]++;
    }
  }
  if (trip_value[0] > 9)
    trip_value[0] = 0;
  
  if (flag)
  {
    odo_value[5]++;
    for (i = 5; i>0; i--)
    {
      if (odo_value[i] > 9)
      {
        odo_value[i] = 0;
        odo_value[i-1]++;
      }
    }
    if (odo_value[0] > 9)
      odo_value[0] = 0;  
  }
}

void update_kilometers()
{
  uint16_t pos = 0;
  uint8_t i;
  
  pos =  3 * 12 * 1;
  for (i = 0; i < 6; i++)
  {
    pos += LCD_bDig(192, pos, odo_value[i], 3, 3, 12, ILI9341_WHITE, ILI9341_BLACK);
  }
  pos = 3 * 12 * 2;
  
  for (i = 0; i < 5; i++)
  {
    if (i < 4)
      pos += LCD_bDig(144, pos, trip_value[i], 3, 3, 12,  ILI9341_WHITE, ILI9341_BLACK);
    else
      pos += LCD_bDig(144, pos, trip_value[i], 3, 3, 12,  ILI9341_BLACK, ILI9341_WHITE);
  }
}

void update_values()
{
  uint16_t pos = 0;
  uint8_t i;
  
  pos = 0;
  for (i = 0; i < 4; i++)
  {
    pos += LCD_bDig(192, pos, odo_value[i], 3, 3, 12, ILI9341_WHITE, ILI9341_BLACK);
  }
  pos = 0;
  
  for (i = 0; i < 4; i++)
  {
    pos += LCD_bDig(144, pos, trip_value[i], 3, 3, 12,  ILI9341_WHITE, ILI9341_BLACK);
  }
}

void predisplay()
{
  
  //uint16_t pos = 178;
  
  //pos += LCD_bDig(0, pos, 4, 3, 3, 12, ILI9341_GREEN, ILI9341_BLACK);
  //pos += LCD_bDig(0, pos, 2, 3, 3, 12, ILI9341_GREEN, ILI9341_BLACK);
  
  //      
  LCD_bDig(192, 3 * 12 * 7, 13, 3, 3, 12, ILI9341_BLUE, ILI9341_BLACK);
  LCD_bDig(192, 3 * 12 * 8, 14, 3, 3, 7, ILI9341_BLUE, ILI9341_BLACK);
  LCD_bDig(144, 0, 15, 3, 3, 12, ILI9341_BLUE, ILI9341_BLACK);
  LCD_bDig(144, 3 * 12 * 1, 16, 3, 3, 12, ILI9341_BLUE, ILI9341_BLACK); 
  LCD_bDig(144, 3 * 12 * 7, 13, 3, 3, 12, ILI9341_BLUE, ILI9341_BLACK);
  LCD_bDig(144, 3 * 12 * 8, 14, 3, 3, 7, ILI9341_BLUE, ILI9341_BLACK);  
  
  //update_kilometers();
  /*
  
  uint16_t pos = 224;
  for (i = 0; i < 6; i++)
  {
  LCD_bDig(pos, 308, 17, 1, 1, 12, ILI9341_GREEN, ILI9341_BLACK);
  pos -= 16;
}
  for (i = 0; i < 6; i++)
  {
  LCD_bDig(pos, 308, 17, 1, 1, 12, ILI9341_YELLOW, ILI9341_BLACK);
  pos -= 16;
}
  for (i = 0; i < 3; i++)
  {
  LCD_bDig(pos, 308, 17, 1, 1, 12, ILI9341_RED, ILI9341_BLACK);
  pos -= 16;
}
  */
  
  LCD_bDig(0, 256, 22, 4, 4, 12, ILI9341_BLUE, ILI9341_BLACK);
  
  LCD_bDig(0, 126, 11, 3, 3, 12, ILI9341_BLUE, ILI9341_BLACK); //  
}


/*
   40.     5 .

*/
#define LPG_LEVEL_CAL_SIZE 9
#define LPG_LEVEL_RESERVE_AMOUNT 10

uint16_t lpg_level_cal[] = {0, 128, 256, 384, 512, 640, 768, 896, 1024 };
uint8_t lpg_value[] = {0, 5, 10, 15, 20, 25, 30, 35, 40};

#define LPG_CALIBRATION_MODE

void update_lpg_level()
{
  //       ,    
  __disable_interrupt();
  
  uint8_t i, index = LPG_LEVEL_CAL_SIZE - 2;
  for (i = 0; i <= index; i++)
  {
    if ((lpg_level >= lpg_level_cal[i]) && (lpg_level <= lpg_level_cal[i+1]))
    {
      index = i;
      break;
    }
  }
  //   .      
  uint16_t diff1 = lpg_level_cal[index+1] - lpg_level_cal[index];
  uint16_t diff2 = ((uint16_t) lpg_value[index+1]) - ((uint16_t) lpg_value[index]);
  diff2 *= 1000;
  diff2 = diff2 / diff1;
  diff1 = (lpg_level - lpg_level_cal[index]) * diff2 + ((uint16_t)lpg_value[index]) * 1000;
  
  uint8_t ltr = (uint8_t) (diff1 / 1000);
  
  uint16_t lpg_color = ILI9341_GREEN;
  if (ltr <= LPG_LEVEL_RESERVE_AMOUNT)
    lpg_color = ILI9341_YELLOW;
  
  uint8_t ltr1 = ltr / 10;
  ltr = ltr - ltr1 * 10;   
  
#ifdef LPG_CALIBRATION_MODE
  uint16_t pos = 162;
  uint16_t div = 1000;
  uint16_t _lpg_level = lpg_level;
  uint16_t _lpg_level1;
  for (i = 0; i < 4; i++)
  {
    _lpg_level1 = _lpg_level / div;
    pos += LCD_bDig(0, pos, (uint8_t)(_lpg_level1), 3, 3, 12, ILI9341_ORANGE, ILI9341_BLACK);
    _lpg_level = _lpg_level - _lpg_level1 * div;
    div = div / 10;
  }
#else  
  uint16_t pos = 178;
  pos += LCD_bDig(0, pos, ltr1, 3, 3, 12, lpg_color, ILI9341_BLACK);
  pos += LCD_bDig(0, pos, ltr, 3, 3, 12, lpg_color, ILI9341_BLACK);
#endif
  // ,    .
  diff2 = (uint16_t)lpg_value[LPG_LEVEL_CAL_SIZE - 1];
  diff2 = diff2 * 1000;
  diff2 = diff2 / 15;
  uint16_t pnt_level = 0;
  
  lpg_color = ILI9341_RED; //   
  pos = 0;
  
#ifdef LPG_CALIBRATION_MODE
  //        
  diff2 = 682; // 682 = (1024 * 10 ) / 15
  diff1 = lpg_level * 10;  
#endif
  
  for (i = 0; i < 15; i++)
  {
    if (pnt_level >= diff1) //   
      lpg_color = ILI9341_BLACK; 
    
    if ((i == 2)  & (lpg_color != ILI9341_BLACK))
      lpg_color = ILI9341_YELLOW;
    if ((i == 8)  & (lpg_color != ILI9341_BLACK))
      lpg_color = ILI9341_GREEN;
    LCD_bDig(pos, 308, 17, 1, 1, 12, lpg_color, ILI9341_BLACK);
    pos += 16;
    pnt_level += diff2;
    
  }
  
  __enable_interrupt();
  
}

void update_voltage()
{
  //if (voltage == 0)
  //  return;
  
  uint8_t i = 0;
  uint16_t _tmp = 0;
  for (i = 0; i < 16; i++)
  {
    if (voltages_adc[i] == 0)
    {
      if (i == 0)
        return;
      break;
    }
    _tmp += voltages_adc[i];
  }
  voltage = _tmp / i;
  
  uint32_t slv_volt = (uint32_t) voltage;
  slv_volt *= VOLTAGE_CONSTANT;
  slv_volt = slv_volt / 10000;
  
  uint8_t _voltage = (uint8_t) slv_volt;
  //uint8_t i;
  uint16_t volt_color = ILI9341_RED;
  for (i = 0; i < sizeof(voltages); i++)
  {
    if (_voltage >= voltages[i])
    {
      volt_color = voltages_colors[i];
      break;
    }
  }  
  
  uint16_t pos = 0;
  uint8_t div = 100;
  uint8_t _value = 0;
  for (i = 0; i < 3; i++)
  {
    _value = _voltage / div;
    _voltage -= _value * div;
    div = div / 10;
    if ((i == 0) && (_value == 0))
      pos += LCD_bDig(0, pos, ILI_SPACE, 3, 3, 12, volt_color, ILI9341_BLACK);
    else
      pos += LCD_bDig(0, pos, _value, 3, 3, 12, volt_color, ILI9341_BLACK);
    if (i == 1)
      pos += LCD_bDig(0, pos, 12, 3, 3, 6,  volt_color, ILI9341_BLACK);
  }    
}

void update_speed()
{
  uint16_t temp, temp1, pos, y;
  uint8_t flag = 1;
  temp = current_speed;
  temp1 = temp / 100;
  y = 56; pos = 3 * 12 * 2; 
  if (temp1 == 0)
    pos += LCD_bDig(y, pos, ILI_SPACE, 5, 5, 12, ILI9341_WHITE, ILI9341_BLACK);
  else
  {
    pos += LCD_bDig(y, pos, (uint8_t) temp1, 5, 5, 12, ILI9341_WHITE, ILI9341_BLACK); 
    flag = 0;
  }
  
  temp = temp - temp1 * 100;
  temp1 = temp / 10;
  
  if ((temp1 == 0) && (flag))
    pos += LCD_bDig(y, pos, ILI_SPACE, 5, 5, 12, ILI9341_WHITE, ILI9341_BLACK);
  else
    pos += LCD_bDig(y, pos, (uint8_t) temp1, 5, 5, 12, ILI9341_WHITE, ILI9341_BLACK); 
  
  temp = temp - temp1 * 10;
  pos += LCD_bDig(y, pos, (uint8_t) temp, 5, 5, 12, ILI9341_WHITE, ILI9341_BLACK); 
  
}

static NO_REG_SAVE void user_thread_func (void)
{
  LCD_Init();
  predisplay();
  update_kilometers();
  
  while (1)
  {    
    if (speed_cnt > IMPULSES_PER_100M)
    {
      speed_cnt-= IMPULSES_PER_100M;
      increase_trip();
      update_kilometers();
      
      ignore_speed_flag = 1;
      ee_Update_Values(odometer, trip_km, speed_cnt);
    }
    else
    {
      if (last_speed_tmr != 0xFFFF)
      {
        uint16_t spd = 22560 / last_speed_tmr;
        if (spd != current_speed)
        {
          //  .
          current_speed = spd;
          update_speed();
        }
      }
      else
      {
        if (current_speed > 0)
        {
          current_speed = 0;
          update_speed();
        }
      }
    }
    update_voltage();
    update_lpg_level();
    
    if (buttons.JustPressed & BUTTON_1)
    {
      buttons.JustPressed &= BUTTON_1_MASK;
      trip_km = 0;
      update_kilometers();
      ee_Update_Values(odometer, trip_km, speed_cnt);
    }
    krn_sleep(100);
  }
}


int main( void )
{
  init_hardware();
  //LCD_Init();
  //fillScreen(ILI9341_BLUE);
  //while (1);
  krn_tmp_stack();
  krn_thread_init();
  krn_thread_create(&thr_user, (void*)user_thread_func, (void*)3, 3, user_thread_stack, MAIN_THREAD_STACK);
  krn_thread_create(&thr_button, (void*)button_thread_func, (void*)3, 3, button_thread_stack, MAIN_THREAD_STACK);
  __enable_interrupt();  
  krn_run();
}
